@@ -200,13 +200,18 @@ inferCompleteAbundance <- function(x) {
 #' \code{countClones} determines the number of sequences and total copy number of 
 #' clonal groups.
 #'
-#' @param    data    data.frame with Change-O style columns containing clonal assignments.
+#' @param    data    data.frame with columns containing clonal assignments.
 #' @param    groups  character vector defining \code{data} columns containing grouping 
 #'                   variables. If \code{groups=NULL}, then do not group data.
 #' @param    copy    name of the \code{data} column containing copy numbers for each 
 #'                   sequence. If this value is specified, then total copy abundance
 #'                   is determined by the sum of copy numbers within each clonal group.
 #' @param    clone   name of the \code{data} column containing clone identifiers.
+#' @param    cell_id name of the \code{data} column containing cell identifiers. If
+#'                   \code{cell_id} column is not present the function will assume bulk data.
+#' @param    remove_na    removes rows with \code{NA} values in the clone column if \code{TRUE} and issues a warning. 
+#'                        Otherwise, keeps those rows and considers \code{NA} as a clone in the final counts 
+#'                        and relative abundances.
 #' 
 #' @return   A data.frame summarizing clone counts and frequencies with columns:
 #'           \itemize{
@@ -235,27 +240,70 @@ inferCompleteAbundance <- function(x) {
 #' clones <- countClones(ExampleDb, groups=c("sample_id", "c_call"), copy="duplicate_count")
 #' 
 #' @export
-countClones <- function(data, groups=NULL, copy=NULL, clone="clone_id") {
+countClones <- function(data, groups=NULL, copy=NULL, clone="clone_id", cell_id="cell_id", 
+                        remove_na=TRUE) {
     # Check input
     check <- checkColumns(data, c(clone, copy, groups))
-    if (check != TRUE) { stop(check) }
-    
+    if (check != TRUE) { 
+        warning(check) # instead of throwing an error and potentially disrupting a workflow
+    }
+
+    # Don't allow a copy column for single-cell data as it doesn't make sense to count copies
+    if (cell_id %in% names(data) & !is.null(copy)) {
+        stop("Copy column specification not allowed for single-cell or mixed bulk and single-cell data. 
+        A cell_id column is present in the dataframe single-cell or mixed bulk and single-cell data is assumed.")
+    }
+
+    # Handle NAs
+    if (remove_na) {
+        bool_na <- is.na(data[, clone])
+        if (any(bool_na)) {
+            if  (!all(bool_na)){
+                msg <- paste0("NA(s) found in ", sum(bool_na), " row(s) of the ", clone, 
+                            " column and excluded from tabulation")
+                warning(msg)
+            }
+            data <- data[!bool_na, ]
+        }
+    }
+
     # Tabulate clonal abundance
-    if (is.null(copy)) {
+    if (cell_id %in% names(data)) {
+        # Handle single-cell and mixed bulk and single-cell case
+        data_sc <- data %>% dplyr::filter(!is.na(!!rlang::sym(cell_id)))
+        data_sc[[cell_id]] <- as.character(data_sc[[cell_id]])
+        data_blk <- data %>% dplyr::filter(is.na(!!rlang::sym(cell_id)))
+        if (nrow(data_blk > 0)){
+            data_blk[[cell_id]] <- paste0("bulk", 1:nrow(data_blk)) #dummy cell_id for bulk data
+        }
+        data <- bind_rows(data_sc, data_blk)
+
         clone_tab <- data %>% 
-            group_by(!!!rlang::syms(c(groups, clone))) %>%
+            dplyr::select(!!!rlang::syms(c(groups, clone, cell_id))) %>%
+            dplyr::distinct() %>%
+            dplyr::group_by(!!!rlang::syms(c(groups, clone))) %>%
             dplyr::summarize(seq_count=n()) %>%
             dplyr::mutate(seq_freq=!!rlang::sym("seq_count")/sum(!!rlang::sym("seq_count"), na.rm=TRUE)) %>%
             dplyr::arrange(desc(!!rlang::sym("seq_count")))
     } else {
-        clone_tab <- data %>% 
-            group_by(!!!rlang::syms(c(groups, clone))) %>%
-            dplyr::summarize(seq_count=length(.data[[clone]]),
-                              copy_count=sum(.data[[copy]], na.rm=TRUE)) %>%
-            dplyr::mutate(seq_freq=!!rlang::sym("seq_count")/sum(!!rlang::sym("seq_count"), na.rm=TRUE),
-                          copy_freq=!!rlang::sym("copy_count")/sum(!!rlang::sym("copy_count"), na.rm=TRUE)) %>%
-            dplyr::arrange(desc(!!rlang::sym("copy_count"))) 
+        if (is.null(copy)) {
+            clone_tab <- data %>% 
+                dplyr::group_by(!!!rlang::syms(c(groups, clone))) %>%
+                dplyr::summarize(seq_count=n()) %>%
+                dplyr::mutate(seq_freq=!!rlang::sym("seq_count")/sum(!!rlang::sym("seq_count"), na.rm=TRUE)) %>%
+                dplyr::arrange(desc(!!rlang::sym("seq_count")))
+
+        } else {
+            clone_tab <- data %>% 
+                dplyr::group_by(!!!rlang::syms(c(groups, clone))) %>%
+                dplyr::summarize(seq_count=length(.data[[clone]]),
+                                copy_count=sum(.data[[copy]], na.rm=TRUE)) %>%
+                dplyr::mutate(seq_freq=!!rlang::sym("seq_count")/sum(!!rlang::sym("seq_count"), na.rm=TRUE),
+                            copy_freq=!!rlang::sym("copy_count")/sum(!!rlang::sym("copy_count"), na.rm=TRUE)) %>%
+                dplyr::arrange(desc(!!rlang::sym("copy_count"))) 
+        }
     }
+
     return(clone_tab)
 }
 
@@ -273,7 +321,7 @@ countClones <- function(data, groups=NULL, copy=NULL, clone="clone_id") {
 bootstrapAbundance <- function(x, n, nboot=200, method="before") {
     ## DEBUG
     # x=abund_obs; method="before"
-    # Check argumets
+    # Check arguments
     method <- match.arg(method)
   
     if (method == "before") {
@@ -329,6 +377,8 @@ bootstrapAbundance <- function(x, n, nboot=200, method="before") {
 #'                     be resampled to its original size or, if specified, \code{max_size}.
 #' @param    ci        confidence interval to calculate; the value must be between 0 and 1.
 #' @param    nboot     number of bootstrap realizations to generate.
+#' @param    cell_id name of the \code{data} column containing cell identifiers. If
+#'                     \code{cell_id=NULL} then the function will assume bulk data.
 #' @param    progress  if \code{TRUE} show a progress bar. 
 #' 
 #' @return   A \link{AbundanceCurve} object summarizing the abundances.
@@ -355,14 +405,25 @@ bootstrapAbundance <- function(x, n, nboot=200, method="before") {
 #' @export
 estimateAbundance <- function(data, clone="clone_id", copy=NULL, group=NULL, 
                               min_n=30, max_n=NULL, uniform=TRUE, ci=0.95, nboot=200,
-                              progress=FALSE) {
+                              cell_id="cell_id", progress=FALSE) {
+    
+    # TODO:
+    # Add alakazam style cell_id=NULL,
+    # similar to distToNearest
+    # filter based on locusValues
+    # if cell_id
+    #    for rows that have unique cell_id, ok
+    #    if rows have cell_id not unique, count only once
+    # if not cell_id, count heavy chains (locusValues will be IGH)
+    # If mixed bulk and sc do calculation but raise warning because different type of abundances
+    
     ## DEBUG
     # data=ExampleDb; group="sample_id"; clone="clone_id"; copy=NULL; min_n=1; max_n=NULL; ci=0.95; uniform=F; nboot=100
     # copy="duplicate_count"
     # group=NULL
 
     # Hack for visibility of dplyr variables
-    . <- NULL
+    #. <- NULL
     
     # Check input
     if (!is.data.frame(data)) {
@@ -379,7 +440,8 @@ estimateAbundance <- function(data, clone="clone_id", copy=NULL, group=NULL,
     
     # Tabulate clonal abundance
     count_col <- if (!is.null(copy)) { "copy_count" } else { "seq_count" }
-    clone_tab <- countClones(data, copy=copy, clone=clone, groups=group) %>%
+    clone_tab <- countClones(data, copy=copy, clone=clone, groups=group, 
+                                cell_id=cell_id) %>%
         dplyr::mutate(clone_count=!!rlang::sym(count_col))
 
     # Tabulate group sizes
@@ -456,7 +518,8 @@ estimateAbundance <- function(data, clone="clone_id", copy=NULL, group=NULL,
         
         if (progress) { pb$tick() }
     }
-    id_col <- if_else(is.null(group), "group", group)
+    id_col <- "group"
+    if (!is.null(group)) { id_col <- group }
     abundance_df <- as.data.frame(bind_rows(abund_list, .id=id_col))
     bootstrap_df <- as.data.frame(bind_rows(boot_list, .id=id_col))
     
@@ -730,7 +793,7 @@ helperBeta <- function(boot_output, q, ci_x, clone="clone_id", group="group") {
 # @return   data.frame containing test results for each value of q.
 helperTest <- function(div_df, q, group="group") {
     # Hack for visibility of dplyr variables
-    . <- NULL
+    #. <- NULL
     
     # Pairwise test
     group_pairs <- combn(unique(div_df[[group]]), 2, simplify=F)
@@ -1043,7 +1106,7 @@ betaDiversity <- function(data, comparisons, min_q=0, max_q=4, step_q=0.1, ci=0.
 
 #### Plotting functions ####
 
-#' Plots a clonal abundance distribution
+#' Plot a clonal abundance distribution
 #' 
 #' \code{plotAbundanceCurve} plots the results from estimating the complete clonal 
 #' relative abundance distribution. The distribution is plotted as a log rank abundance 
@@ -1056,9 +1119,12 @@ betaDiversity <- function(data, comparisons, min_q=0, max_q=4, step_q=0.1, ci=0.
 #' @param    main_title    string specifying the plot title.
 #' @param    legend_title  string specifying the legend title.
 #' @param    xlim          numeric vector of two values specifying the 
-#'                         \code{c(lower, upper)} x-axis limits.
+#'                         \code{c(lower, upper)} x-axis limits. The lower x-axis 
+#'                         value must be >=1.
 #' @param    ylim          numeric vector of two values specifying the 
-#'                         \code{c(lower, upper)} y-axis limits.
+#'                         \code{c(lower, upper)} y-axis limits. The limits on the 
+#'                         abundance values are expressed as fractions of 1: use
+#'                         c(0,1) to set the lower and upper limits to 0\% and 100\%.
 #' @param    annotate      string defining whether to added values to the group labels 
 #'                         of the legend. When \code{"none"} (default) is specified no
 #'                         annotations are added. Specifying (\code{"depth"}) adds 
@@ -1066,13 +1132,12 @@ betaDiversity <- function(data, comparisons, min_q=0, max_q=4, step_q=0.1, ci=0.
 #' @param    silent        if \code{TRUE} do not draw the plot and just return the ggplot2 
 #'                         object; if \code{FALSE} draw the plot.
 #' @param    ...           additional arguments to pass to ggplot2::theme.
-#'
+#' 
 #' @return   A \code{ggplot} object defining the plot.
 #' 
 #' @seealso  
-#' See \link{AbundanceCurve} for the input object and \link{estimateAbundance} for 
-#' generating the input abundance distribution.
-#' Plotting is performed with \link{ggplot}.
+#' See \link{AbundanceCurve} for the input object and \link{estimateAbundance} for
+#' generating the input abundance distribution. Plotting is performed with \link[ggplot2]{ggplot}.
 #'           
 #' @examples
 #' # Estimate abundance by sample and plot
@@ -1088,6 +1153,17 @@ plotAbundanceCurve <- function(data, colors=NULL, main_title="Rank Abundance",
     # Check if abundance is in data
     if (is.null(data@abundance)) { stop("Missing abundance data.") }
     
+    # Validate abundance limits
+    if (!is.null(xlim)) {
+        if (xlim[1]<1) {
+            stop("The lower x-axis xlim value must be >=1.")
+        }
+        max_xlim <- max(data@abundance$rank, na.rm = T)
+        if (xlim[2]>max_xlim) {
+            message("The largest x-axis value is ",max_xlim,".")
+        }
+    }    
+
     # Check arguments
     annotate <- match.arg(annotate)
     
@@ -1106,17 +1182,21 @@ plotAbundanceCurve <- function(data, colors=NULL, main_title="Rank Abundance",
     
     if (!all(is.na(group_labels))) {
         # Define grouped plot
-        p1 <- ggplot(data@abundance, aes_string(x="rank", y="p", group=data@group_by)) + 
+        p1 <- ggplot(data@abundance, aes(x=!!rlang::sym("rank"), 
+                                         y=!!rlang::sym("p"), 
+                                         group=!!rlang::sym(data@group_by))) + 
             ggtitle(main_title) + 
             baseTheme() + 
             xlab("Rank") +
             ylab("Abundance") +
-            scale_x_log10(limits=xlim,
+            scale_x_log10(
                           breaks=scales::trans_breaks("log10", function(x) 10^x),
                           labels=scales::trans_format("log10", scales::math_format(10^.x))) +
             scale_y_continuous(labels=scales::percent) +
-            geom_ribbon(aes_string(ymin="lower", ymax="upper", fill=data@group_by), alpha=0.4) +
-            geom_line(aes_string(color=data@group_by))
+            geom_ribbon(aes(ymin=!!rlang::sym("lower"), 
+                            ymax=!!rlang::sym("upper"), 
+                            fill=!!rlang::sym(data@group_by)), alpha=0.4) +
+            geom_line(aes(color=!!rlang::sym(data@group_by)))
         
         # Set colors and legend
         if (!is.null(colors)) {
@@ -1134,21 +1214,25 @@ plotAbundanceCurve <- function(data, colors=NULL, main_title="Rank Abundance",
             line_color <- "black"
         }
         # Define plot
-        p1 <- ggplot(data@abundance, aes_string(x="rank", y="p")) + 
+        p1 <- ggplot(data@abundance, aes(x=!!rlang::sym("rank"), 
+                                         y=!!rlang::sym("p"))) + 
             ggtitle(main_title) + 
             baseTheme() + 
             xlab("Rank") +
             ylab("Abundance") +
-            scale_x_log10(limits=xlim,
+            scale_x_log10(
                           breaks=scales::trans_breaks("log10", function(x) 10^x),
                           labels=scales::trans_format("log10", scales::math_format(10^.x))) +
             scale_y_continuous(labels=scales::percent) +
-            geom_ribbon(aes_string(ymin="lower", ymax="upper"), fill=line_color, alpha=0.4) +
+            geom_ribbon(aes(ymin=!!rlang::sym("lower"), 
+                            ymax=!!rlang::sym("upper")), 
+                        fill=line_color, alpha=0.4) +
             geom_line(color=line_color)
     }
-    
     # Add additional theme elements
-    p1 <- p1 + do.call(theme, list(...))
+    p1 <- p1 + 
+        coord_cartesian(xlim=xlim,ylim=ylim) +
+        do.call(theme, list(...))
     
     # Plot
     if (!silent) { plot(p1) }
@@ -1189,7 +1273,7 @@ plotAbundanceCurve <- function(data, colors=NULL, main_title="Rank Abundance",
 #' @return   A \code{ggplot} object defining the plot.
 #' 
 #' @seealso  See \link{alphaDiversity} and \link{alphaDiversity} for generating 
-#'           \link{DiversityCurve} objects for input. Plotting is performed with \link{ggplot}.
+#'           \link{DiversityCurve} objects for input. Plotting is performed with \link[ggplot2]{ggplot}.
 #' 
 #' @examples
 #' # Calculate diversity
@@ -1198,7 +1282,7 @@ plotAbundanceCurve <- function(data, colors=NULL, main_title="Rank Abundance",
 #' # Plot diversity
 #' plotDiversityCurve(div, legend_title="Sample")
 #'
-#' #' # Plot diversity
+#' # Plot diversity
 #' plotDiversityCurve(div, legend_title="Sample", score="evenness")
 #' 
 #' @export
@@ -1239,13 +1323,15 @@ plotDiversityCurve <- function(data, colors=NULL, main_title="Diversity",
     
     if (!all(is.na(group_labels))) {
         # Define grouped plot
-        p1 <- ggplot(data@diversity, aes_string(x="q", y=y_value, group=data@group_by)) + 
+        p1 <- ggplot(data@diversity, aes(x=q, y=!!rlang::sym(y_value), 
+                                         group=!!rlang::sym(data@group_by))) + 
             ggtitle(main_title) + 
             baseTheme() + 
             xlab('q') +
             ylab(y_label) +
-            geom_ribbon(aes_string(ymin=y_min, ymax=y_max, fill=data@group_by), alpha=0.4) +
-            geom_line(aes_string(color=data@group_by))
+            geom_ribbon(aes(ymin=!!rlang::sym(y_min), ymax=!!rlang::sym(y_max), 
+                            fill=!!rlang::sym(data@group_by)), alpha=0.4) +
+            geom_line(aes(color=!!rlang::sym(data@group_by)))
     
         # Set colors and legend
         if (!is.null(colors)) {
@@ -1264,35 +1350,37 @@ plotDiversityCurve <- function(data, colors=NULL, main_title="Diversity",
         }
       
         # Define ungrouped plot
-        p1 <- ggplot(data@diversity, aes_string(x="q", y=y_value)) + 
+        p1 <- ggplot(data@diversity, aes(x=q, y=!!rlang::sym(y_value))) + 
             ggtitle(main_title) + 
             baseTheme() + 
             xlab('q') +
             ylab(y_label) +
-            geom_ribbon(aes_string(ymin=y_min, ymax=y_max), fill=line_color, alpha=0.4) +
+            geom_ribbon(aes(ymin=!!rlang::sym(y_min), ymax=!!rlang::sym(y_max)), fill=line_color, alpha=0.4) +
             geom_line(color=line_color)
     }
     
     # Set x-axis style
     if (log_x) {
-        p1 <- p1 + scale_x_continuous(trans=scales::log2_trans(), limits=xlim,
+        p1 <- p1 + scale_x_continuous(trans=scales::log2_trans(),
                                       breaks=scales::trans_breaks('log2', function(x) 2^x),
                                       labels=scales::trans_format('log2', scales::math_format(2^.x)))
     } else {
-        p1 <- p1 + scale_x_continuous(limits=xlim)
+        p1 <- p1 + scale_x_continuous()
     }
     
     # Set y-axis style
     if (log_y) {
-        p1 <- p1 + scale_y_continuous(trans=scales::log2_trans(), limits=ylim,
+        p1 <- p1 + scale_y_continuous(trans=scales::log2_trans(),
                                       breaks=scales::trans_breaks('log2', function(x) 2^x),
                                       labels=scales::trans_format('log2', scales::math_format(2^.x)))
     } else {
-        p1 <- p1 + scale_y_continuous(limits=ylim)
+        p1 <- p1 + scale_y_continuous()
     }
     
     # Add additional theme elements
-    p1 <- p1 + do.call(theme, list(...))
+    p1 <- p1 + 
+        coord_cartesian(xlim=xlim,ylim=ylim) +
+        do.call(theme, list(...))
 
     # Plot
     if (!silent) { plot(p1) }
@@ -1327,7 +1415,7 @@ plotDiversityCurve <- function(data, colors=NULL, main_title="Diversity",
 #' @return   A \code{ggplot} object defining the plot.
 #' 
 #' @seealso  See \link{alphaDiversity} for generating input.
-#'           Plotting is performed with \link{ggplot}.
+#'           Plotting is performed with \link[ggplot2]{ggplot}.
 #' 
 #' @examples
 #' # Calculate diversity
@@ -1372,13 +1460,15 @@ plotDiversityTest <- function(data, q, colors=NULL, main_title="Diversity", lege
                       upper=!!rlang::sym("d") + !!rlang::sym("d_sd"))
     
     # Define base plot elements
-    p1 <- ggplot(df, aes_string(x=data@group_by)) + 
+    p1 <- ggplot(df, aes(x=!!rlang::sym(data@group_by))) + 
         ggtitle(main_title) + 
         baseTheme() + 
         xlab("") +
         ylab(bquote("Mean " ^ .(q) * D %+-% "SD")) +
-        geom_linerange(aes_string(ymin="lower", ymax="upper", color=data@group_by), alpha=0.8) +
-        geom_point(aes_string(y="d", color=data@group_by))
+        geom_linerange(aes(ymin=!!rlang::sym("lower"), 
+                           ymax=!!rlang::sym("upper"), 
+                           color=!!rlang::sym(data@group_by)), alpha=0.8) +
+        geom_point(aes(y=!!rlang::sym("d"), color=!!rlang::sym(data@group_by)))
     
     # Set colors and legend
     if (!is.null(colors)) {
